// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "SizeHeuristic.hpp"
#include "GridPoint.hpp"
#include "fastbits.hpp"

using namespace FastBits;

DlxHead* SizeHeuristic::selectDlx() const
{
    DlxHead* dlxRoot = (DlxHead*) entity->dlxRoot;
    DlxHead* best    = (DlxHead*) dlxRoot->right;
    for(DlxHead* head = (DlxHead*) best->right;
            head != dlxRoot; head = (DlxHead*) head->right)
    {
        if(head->numRow < best->numRow)
            best = head;
    }
    return best;
}


ImageListSet SizeHeuristic::selectFila(int nrp, int nrs, const gridbitfield_t occupancy, bool dlxPristine) const
{
    if(dlxPristine)
    {
        GridPoint* best = (GridPoint*) entity->dlxRoot->right;
        for(DlxHead* head = (DlxHead*) best->right;
                head->isGridPoint(); head = (DlxHead*) head->right)
        {
            GridPoint* gp = (GridPoint*) head;
            if(gp->numRow < best->numRow)
                best = gp;
        }

        if(best->numRow == 0)
            return NULL;
        else
            return entity->getImageListSet(best, occupancy);
    }
    else
    {
        gridbitfield_t holes = ~occupancy;
        int minFits = 0x1fffffff;   // just a big number
        ImageListSet best = NULL;
        for(int b = lowestSetBit(holes); b != -1 ; b = nextLowestSetBit(holes, b))
        {
            ImageListSet imageListSet = entity->getImageListSet(entity->filaGrid[b], occupancy);
            int numFits = countFits(
                    imageListSet,
                    entity->remainingShapeList,
                    nrs,
                    occupancy,
                    minFits);

            if(numFits - minFits < 0)  // Written this way to avoid overflow compiler warning
            {
                if(numFits == 0)
                    return NULL;
                minFits = numFits;
                best = imageListSet;
            }
        }
        return best;
    }
}
