// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2010 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef MAKEARRAY_HPP
#define MAKEARRAY_HPP

#include <stdlib.h>
#include <iostream>

template <class T>
inline T* makeArray(
        long long d1,
        long long b1 = 0)
{
    if(d1 == 0)
        return NULL;

    T* m1 = new T[d1] - b1;
    return m1;
}

template <class T>
inline T* initArray(
        long long d1,
        const T& init,
        long long b1 = 0)
{
    T* m1 = makeArray<T>(d1, b1);

    for(long long i1 = b1; i1 < d1 + b1; ++i1)
        m1[i1] = init;
    return m1;
}

template <class T>
inline void deleteArray(
    T* a,
    long long b1 = 0)
{
    if(a == NULL)
        return;

    delete [] (a + b1);
}

template <class T>
inline T** makeArray2(
        long long d1,
        long long d2,
        long long b1 = 0,
        long long b2 = 0)
{
    if(d1 == 0 || d2 == 0)
        return NULL;

    T** m1 = new T*[d1] - b1;
    T* m2 = new T[d1*d2];
    for(long long i1 = b1; i1 < d1 + b1; ++i1)
    {
        m1[i1] = m2 - b2;
        m2 += d2;
    }
    return m1;
}

template <class T>
inline T** initArray2(
        long long d1,
        long long d2,
        const T& init,
        long long b1 = 0,
        long long b2 = 0)
{
    T** m1 = makeArray2<T>(d1, d2, b1, b2);

    for(long long i1 = b1; i1 < d1 + b1; ++i1)
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
            m1[i1][i2] = init;
    return m1;
}

template <class T>
inline void deleteArray2(
    T** a,
    long long b1 = 0,
    long long b2 = 0)
{
    if(a == NULL)
        return;

    delete [] (a[b1] + b2);
    delete [] (a + b1);
}

template <class T>
inline T*** makeArray3(
        long long d1,
        long long d2,
        long long d3,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0)
{
    if(d1 == 0 || d2 == 0 || d3 == 0)
        return NULL;

    T*** m1 = new T**[d1] - b1;
    T** m2 = new T*[d1*d2];
    T* m3 = new T[d1*d2*d3];
    for(long long i1 = b1; i1 < d1 + b1; ++i1)
    {
        m1[i1] = m2 - b2;
        m2 += d2;
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
        {
            m1[i1][i2] = m3 - b3;
            m3 += d3;
        }
    }
    return m1;
}

template <class T>
inline T*** initArray3(
        long long d1,
        long long d2,
        long long d3,
        const T& init,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0)
{
    T*** m1 = makeArray3<T>(d1, d2, d3, b1, b2, b3);

    for(long long i1 = b1; i1 < d1 + b1; ++i1)
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
            for(long long i3 = b3; i3 < d3 + b3; ++i3)
                m1[i1][i2][i3] = init;
    return m1;
}

template <class T>
inline void deleteArray3(
    T*** a,
    long long b1 = 0,
    long long b2 = 0,
    long long b3 = 0)
{
    if(a == NULL)
        return;

    delete [] (a[b1][b2] + b3);
    delete [] (a[b1] + b2);
    delete [] (a + b1);
}

template <class T>
inline T**** makeArray4(
        long long d1,
        long long d2,
        long long d3,
        long long d4,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0,
        long long b4 = 0)
{
    if(d1 == 0 || d2 == 0 || d3 == 0 || d4 == 0)
        return NULL;
    T**** m1 = new T***[d1] - b1;
    T*** m2 = new T**[d1*d2];
    T** m3 = new T*[d1*d2*d3];
    T* m4 = new T[d1*d2*d3*d4];
    for(long long i1 = b1; i1 < d1 + b1; ++i1)
    {
        m1[i1] = m2 - b2;
        m2 += d2;
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
        {
            m1[i1][i2] = m3 - b3;
            m3 += d3;
            for(long long i3 = b3; i3 < d3 + b3; ++i3)
            {
                m1[i1][i2][i3] = m4 - b4;
                m4 += d4;
            }
        }
    }
    return m1;
}

template <class T>
inline T**** initArray4(
        long long d1,
        long long d2,
        long long d3,
        long long d4,
        const T& init,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0,
        long long b4 = 0)
{
    T**** m1 = makeArray4<T>(d1, d2, d3, d4, b1, b2, b3, b4);

    for(long long i1 = b1; i1 < d1 + b1; ++i1)
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
            for(long long i3 = b3; i3 < d3 + b3; ++i3)
                for(long long i4 = b4; i4 < d4 + b4; ++i4)
                    m1[i1][i2][i3][i4] = init;
    return m1;
}

template <class T>
inline void deleteArray4(
    T**** a,
    long long b1 = 0,
    long long b2 = 0,
    long long b3 = 0,
    long long b4 = 0)
{
    if(a == NULL)
        return;

    delete [] (a[b1][b2][b3] + b4);
    delete [] (a[b1][b2] + b3);
    delete [] (a[b1] + b2);
    delete [] (a + b1);
}

template <class T>
inline T***** makeArray5(
        long long d1,
        long long d2,
        long long d3,
        long long d4,
        long long d5,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0,
        long long b4 = 0,
        long long b5 = 0)
{
    if(d1 == 0 || d2 == 0 || d3 == 0 || d4 == 0 || d5 == 0)
        return NULL;
    T***** m1 = new T****[d1] - b1;
    T**** m2 = new T***[d1*d2];
    T*** m3 = new T**[d1*d2*d3];
    T** m4 = new T*[d1*d2*d3*d4];
    T* m5 = new T[d1*d2*d3*d4*d5];
    for(long long i1 = b1; i1 < d1 + b1; ++i1)
    {
        m1[i1] = m2 - b2;
        m2 += d2;
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
        {
            m1[i1][i2] = m3 - b3;
            m3 += d3;
            for(long long i3 = b3; i3 < d3 + b3; ++i3)
            {
                m1[i1][i2][i3] = m4 - b4;
                m4 += d4;
                for(long long i4 = b4; i4 < d4 + b4; ++i4)
                {
                    m1[i1][i2][i3][i4] = m5 - b5;
                    m5 += d5;
                }
            }
        }
    }
    return m1;
}

template <class T>
inline T***** initArray5(
        long long d1,
        long long d2,
        long long d3,
        long long d4,
        long long d5,
        const T& init,
        long long b1 = 0,
        long long b2 = 0,
        long long b3 = 0,
        long long b4 = 0,
        long long b5 = 0)
{
    T***** m1 = makeArray5<T>(d1, d2, d3, d4, d5, b1, b2, b3, b4, b5);

    for(long long i1 = b1; i1 < d1 + b1; ++i1)
        for(long long i2 = b2; i2 < d2 + b2; ++i2)
            for(long long i3 = b3; i3 < d3 + b3; ++i3)
                for(long long i4 = b4; i4 < d4 + b4; ++i4)
                    for(long long i5 = b5; i5 < d5 + b5; ++i5)
                        m1[i1][i2][i3][i4][i5] = init;
    return m1;
}

template <class T>
inline void deleteArray5(
    T***** a,
    long long b1 = 0,
    long long b2 = 0,
    long long b3 = 0,
    long long b4 = 0,
    long long b5 = 0)
{
    if(a == NULL)
        return;

    delete [] (a[b1][b2][b3][b4] + b5);
    delete [] (a[b1][b2][b3] + b4);
    delete [] (a[b1][b2] + b3);
    delete [] (a[b1] + b2);
    delete [] (a + b1);
}

#endif
